﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ship
{
    internal class SetShipsGeneric<T>
        where T: class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly T[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetShipsGeneric(int count)
        {
            _places = new T[count];
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="ship">Добавляемый корабль</param>
        /// <returns></returns>
        public int Insert(T ship)
        {
            return Insert(ship, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="ship">Добавляемый корабль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T ship, int position)
        {
            if (position < 0 || position >= Count)
            {
                return -1;
            }

            if (_places[position] == null)
            {
                _places[position] = ship;
                return position;
            }

            int firstNull = -1;

            for (int i = position + 1; i < Count; i++)
            {
                if (_places[i] == null)
                {
                    firstNull = i;
                    break;
                }
            }

            if (firstNull == -1)
            {
                return -1;
            }

            for (int i = firstNull; i > position; i--)
            {
                (_places[i], _places[i - 1]) = (_places[i - 1], _places[i]);
            }
            _places[position] = ship;
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 || position >= Count || _places[position] == null)
            {
                return null;
            }

            var result = _places[position];

            _places[position] = null;
            return result;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Get(int position)
        {
            if (position < 0 || position >= Count)
            {
                return null;
            }
            return _places[position];
        }

    }
}
